@@ -81,7 +81,12 @@ _orc_compiler_init (void)
 {
   char *envvar;
 #ifdef HAVE_CODEMEM_VIRTUALALLOC
-  size_t page_size;
+  #ifdef ORC_WINAPI_ONLY_APP
+    size_t page_size;
+  #else
+    size_t page_size ORC_GNUC_UNUSED;
+  #endif
+
   SYSTEM_INFO info;
 #endif
 
@@ -96,9 +101,6 @@ _orc_compiler_init (void)
   _orc_compiler_flag_debug = orc_compiler_flag_check ("debug");
   _orc_compiler_flag_randomize = orc_compiler_flag_check ("randomize");
 
-  /* 16 bytes alignment by default */
-  _orc_codemem_alignment = 15;
-
 #ifdef HAVE_CODEMEM_VIRTUALALLOC
   GetNativeSystemInfo(&info);
   page_size = info.dwPageSize;
@@ -109,6 +111,9 @@ _orc_compiler_init (void)
    * memory can be affected by later generated one.
    */
   _orc_codemem_alignment = info.dwPageSize - 1;
+#else
+  /* 16 bytes alignment by default */
+  _orc_codemem_alignment = 15;
 #endif
 
 #ifdef ORC_WINAPI_ONLY_APP
@@ -173,8 +178,10 @@ orc_compiler_allocate_register (OrcCompiler *compiler, int data_reg)
     roff = rand()&0x1f;
   }
 
-  for(i=0;i<32;i++){
-    reg = offset + ((roff + i)&0x1f);
+  for (i = 0; i < ORC_N_REGS; i++) {
+    // For volatile registers, try up to 32 registers from offset
+    // Amyspark: this ensures that we don't run over vector registers
+    reg = offset + ((roff + i) & 0x1f);
     if (compiler->valid_regs[reg] &&
         !compiler->save_regs[reg] &&
         compiler->alloc_regs[reg] == 0) {
@@ -183,8 +190,10 @@ orc_compiler_allocate_register (OrcCompiler *compiler, int data_reg)
       return reg;
     }
   }
-  for(i=0;i<32;i++){
-    reg = offset + ((roff + i)&0x1f);
+  // Use ORC_N_REGS to ensure forward proofed iteration
+  for (i = 0; i < ORC_N_REGS; i++) {
+    // Try with up to 64 registers starting from offset
+    reg = offset + ((roff + i) & 0x3f);
     if (compiler->valid_regs[reg] &&
         compiler->alloc_regs[reg] == 0) {
       compiler->alloc_regs[reg]++;
@@ -828,7 +837,8 @@ orc_compiler_get_temp_reg (OrcCompiler *compiler)
   ORC_DEBUG("at insn %d %s", compiler->insn_index,
       compiler->insns[compiler->insn_index].opcode->name);
 
-  for(j=compiler->min_temp_reg;j<ORC_VEC_REG_BASE+32;j++){
+  // Use ORC_N_REGS to ensure forward proofed iteration
+  for (j = compiler->min_temp_reg; j < ORC_N_REGS; j++) {
     if (compiler->valid_regs[j] && !compiler->alloc_regs[j]) {
       compiler->min_temp_reg = j+1;
       if (compiler->max_used_temp_reg < j) compiler->max_used_temp_reg = j;
